#generic-modal .form-horizontal{
    margin-bottom: 0 !important;
}
    #modal-form .control-group{
        margin-top:10px;
    }

    #modal-form  .alert{
        margin-top:10px;
    }
        #modal-form  .control-group .control-label{
            font-weight: bold;
            text-align:left !important;
            width:200px;
            padding-top:0;
            float: left;
        }
        #modal-form  .control-group .controls{
            margin-left: 200px !important;
            padding-right: 40px;
        }


div.modal-header h2 {
    display: inline-block;
    vertical-align: middle;
    margin: 0px;
    width: 500px;
}

div.modal-body textarea {
    width:100%;
    height: 100px;
}

.modal-body {
    padding: 15px !important;
}

.modal-footer {
    margin-top:0px !important;
}
.modal_icon {
    display:inline-block;
    vertical-align: middle;
    height:36px;
    width:36px;
    background-size:36px;
    background-position: left center;
    background-repeat: no-repeat;
    padding-right:15px;
}
