<form action='{{action}}' method='POST' id='modal-form' class='form-horizontal'>
    <input type='hidden' name='csrfmiddlewaretoken' value='{{ csrf_token_value }}'/>
        {% include form_template_name %}
</form>