{% extends 'django_modalview/modal_content.html' %}

{% block modal-description %}
    {% block description %}
        {% if description %}
            {{ description|safe }}
        {% endif %}
    {% endblock %}
    {% block form %}
        {% if form %}
            {% include 'django_modalview/modal_form.html' %}
        {% endif %}
    {% endblock %}
{% endblock %}